@@ -6,8 +6,10 @@ from inspect import isfunction
 from operator import mul
 from functools import partial, reduce, wraps
 
+from local_attention import LocalAttention
 from axial_positional_embedding import AxialPositionalEmbedding
 from product_key_memory import PKM
+from mixture_of_experts import MoE
 from routing_transformer.reversible import ReversibleSequence, SequentialSequence
 
 # constants
@@ -72,13 +74,6 @@ def scatter_mean(src, t, index, dim, eps = 1e-5):
     denom = src.scatter_add(dim, index, torch.ones_like(t))
     return numer / (denom + eps)
 
-def look_around(x, backward = 1, forward = 0, pad_value = -1, dim = 2):
-    t = x.shape[1]
-    dims = (len(x.shape) - dim) * (0, 0)
-    padded_x = F.pad(x, (*dims, backward, forward), value= pad_value)
-    tensors = [padded_x[:, ind:(ind + t), ...] for ind in range(forward + backward + 1)]
-    return torch.cat(tensors, dim=dim)
-
 def split_at_index(dim, index, t):
     pre_slices = (slice(None),) * dim
     l = (*pre_slices, slice(None, index))
@@ -94,7 +89,7 @@ def ema_inplace(moving_avg, new, decay):
     if is_empty(moving_avg):
         moving_avg.data.copy_(new)
         return
-    moving_avg.data.mul_(decay).add_(1 - decay, new)
+    moving_avg.data.mul_(decay).add_(new, alpha= (1 - decay))
 
 # helper classes
 
@@ -133,121 +128,6 @@ class ProjectInOut(nn.Module):
         x = self.project_out(x)
         return x, loss
 
-# positional embeddings
-
-class AbsolutePositionalEmbedding(nn.Module):
-    def __init__(self, dim, max_seq_len):
-        super().__init__()
-        self.emb = nn.Embedding(max_seq_len, dim)
-
-    def forward(self, x):
-        t = torch.arange(x.shape[1], device=x.device)
-        return self.emb(t)
-
-def shift(x):
-    *_, i, j = x.shape
-    zero_pad = torch.zeros((*_, i, i), **to(x))
-    x = torch.cat([x, zero_pad], -1)
-    l = i + j - 1
-    x = x.view(*_, -1)
-    zero_pad = torch.zeros(*_, -x.size(-1) % l, **to(x))
-    shifted = torch.cat([x, zero_pad], -1).view(*_, -1, l)
-    return shifted[..., :i, i - 1:]
-
-class RelativePositionalEmbedding(nn.Module):
-    def __init__(self, dim, heads, length):
-        super().__init__()
-        self.scale = dim ** -0.5
-        self.weights = nn.Parameter(torch.zeros(length, heads, dim))
-
-    def forward(self, q):
-        emb = torch.einsum('bhnid,jhd->bhnij', q, self.weights.type(q.dtype)) * self.scale
-        return shift(emb)
-
-# local attention
-
-class LocalAttention(nn.Module):
-    def __init__(self, bucket_size, heads, head_dim, causal = False, look_backward = 1, look_forward = None, dropout = 0., shared_qk = False, rel_pos_emb = True):
-        super().__init__()
-        self.look_forward = default(look_forward, 0 if causal else 1)
-        assert not (causal and self.look_forward > 0), 'you cannot look forward if causal'
-
-        self.bucket_size = bucket_size
-        self.causal = causal
-        self.look_backward = look_backward
-        self.shared_qk = shared_qk
-
-        self.heads = heads
-        self.dropout = nn.Dropout(dropout)
-
-        self.rel_pos = RelativePositionalEmbedding(head_dim, heads, bucket_size * 2) if rel_pos_emb else None
-
-    def forward(self, q, k, v, input_mask = None):
-        shape = q.shape
-
-        merge_into_batch = lambda t: t.reshape(-1, *t.shape[-2:])
-        q, k, v = map(merge_into_batch, (q, k, v))
-
-        b, t, e, h, device, dtype = *q.shape, self.heads, q.device, q.dtype
-        bucket_size, causal, look_backward, look_forward, shared_qk = self.bucket_size, self.causal, self.look_backward, self.look_forward, self.shared_qk
-
-        buckets = t // bucket_size
-
-        if shared_qk:
-            k = F.normalize(k, 2, dim=-1).type(q.type())
-
-        ticker = torch.arange(t, device=device, dtype=dtype)[None, :]
-        b_t = ticker.reshape(1, buckets, bucket_size)
-
-        bucket_fn = lambda t: t.reshape(b, buckets, bucket_size, -1)
-        bq, bk, bv = map(bucket_fn, (q, k, v))
-
-        look_around_kwargs = {'backward': look_backward, 'forward': look_forward}
-        bk = look_around(bk, **look_around_kwargs)
-        bv = look_around(bv, **look_around_kwargs)
-
-        bq_t = b_t
-        bq_k = look_around(b_t, **look_around_kwargs)
-
-        dots = torch.einsum('bhie,bhje->bhij', bq, bk) * (e ** -0.5)
-
-        if self.rel_pos is not None:
-            rel_attn = self.rel_pos(bq.view(-1, h, *bq.shape[1:])).reshape_as(dots)
-            dots = dots + rel_attn
-
-        mask_value = max_neg_value(dots)
-
-        if shared_qk:
-            mask = bq_t[:, :, :, None] == bq_k[:, :, None, :]
-            dots.masked_fill_(mask, TOKEN_SELF_ATTN_VALUE)
-            del mask
-
-        if causal:
-            mask = bq_t[:, :, :, None] < bq_k[:, :, None, :]
-            dots.masked_fill_(mask, mask_value)
-            del mask
-
-        mask = bq_k[:, :, None, :] == -1
-        dots.masked_fill_(mask, mask_value)
-        del mask
-
-        if input_mask is not None:
-            h = b // input_mask.shape[0]
-            input_mask = input_mask.reshape(-1, buckets, bucket_size)
-            mq = mk = input_mask
-            mk = look_around(mk, pad_value=False, **look_around_kwargs)
-            mask = (mq[:, :, :, None] * mk[:, :, None, :])
-            mask = merge_dims(0, 1, expand_dim(mask, 1, h))
-            dots.masked_fill_(~mask, mask_value)
-            del mask
-
-        attn = dots.softmax(dim=-1)
-        attn = self.dropout(attn)
-
-        out = torch.einsum('bhij,bhje->bhie', attn, bv)
-        out = out.reshape(*shape)
-        return out
-
 # kmeans related function and class
 
 def update_kmeans_on_backwards(module):
@@ -292,8 +172,7 @@ def kmeans_iter(x, means, buckets = None):
 
 def distribution(dists, window_size):
     _, topk_indices = dists.topk(k=window_size, dim=-2)
-    sort_val, _ = topk_indices.sort(dim=-2)
-    indices = sort_val.transpose(-2, -1)
+    indices = topk_indices.transpose(-2, -1)
     return indices.reshape(*indices.size()[:2], -1)
 
 class Kmeans(nn.Module):
@@ -313,10 +192,16 @@ class Kmeans(nn.Module):
             return
         _, h, _, d, device, dtype = *x.shape, x.device, x.dtype
 
-        means = self.means.type(dtype)
-        num_clusters = means.shape[1]
+        num_clusters = self.means.shape[1]
+
         means = x.transpose(0, 1).contiguous().view(h, -1, d)
-        indices = torch.randperm(means.size(1), device=device)[:num_clusters]
+        num_samples = means.shape[1]
+
+        if num_samples >= num_clusters:
+            indices = torch.randperm(num_samples, device=device)[:num_clusters]
+        else:
+            indices = torch.randint(0, num_samples, (num_clusters,), device=device)
+
         means = means[:, indices]
 
         for _ in range(KMEAN_INIT_ITERS):
@@ -360,7 +245,7 @@ class Kmeans(nn.Module):
 # kmeans attention class
 
 class KmeansAttention(nn.Module):
-    def __init__(self, num_clusters, window_size, num_heads, head_dim, causal = False, dropout = 0., ema_decay = 0.999, commitment = 1e-4, context_window_size = None, shared_qk = True):
+    def __init__(self, num_clusters, window_size, num_heads, head_dim, causal = False, dropout = 0., ema_decay = 0.999, commitment = 1e-4, context_window_size = None, receives_context = False, num_mem_kv = 0, shared_qk = False):
         super().__init__()
         self.num_heads = num_heads
         self.num_clusters = num_clusters
@@ -371,9 +256,14 @@ class KmeansAttention(nn.Module):
         self.causal = causal
 
         self.shared_qk = shared_qk
+        self.receives_context = receives_context
         self.kmeans = Kmeans(num_heads, head_dim, num_clusters, ema_decay, commitment)
         self.dropout = nn.Dropout(dropout)
 
+        self.num_mem_kv = max(num_mem_kv, 1 if causal and not shared_qk else 0)
+        self.mem_key = nn.Parameter(torch.randn(num_heads, num_clusters, self.num_mem_kv, head_dim))
+        self.mem_value = nn.Parameter(torch.randn(num_heads, num_clusters, self.num_mem_kv, head_dim))
+
     def forward(self, q, k, v, query_mask = None, key_mask = None, **kwargs):
         b, h, t, d, kv_t, wsz, c_wsz, nc, device, dtype = *q.shape, k.shape[2], self.window_size, self.context_window_size, self.num_clusters, q.device, q.dtype
         is_reverse = kwargs.pop('_reverse', False)
@@ -381,22 +271,23 @@ class KmeansAttention(nn.Module):
         out = torch.zeros_like(q, dtype=dtype)
 
         update_kmeans = self.training and not is_reverse
+        
+        key_mask = default(key_mask, query_mask) if not self.receives_context else key_mask
+        kv_wsz = wsz if not self.receives_context else c_wsz
 
-        if self.shared_qk:
-            wsz = min(wsz, t)
-            dists, aux_loss = self.kmeans(k, update_kmeans)
-            indices = distribution(dists, wsz)
-            k = F.normalize(k, 2, dim=-1).type(dtype)
-            key_mask = query_mask
-            kv_indices = indices
-        else:
-            wsz = min(wsz, t)
-            c_wsz = min(c_wsz, kv_t)
+        wsz = min(wsz, t)
+        kv_wsz = min(kv_wsz, kv_t)
 
+        if not self.shared_qk or self.receives_context:
             dists, aux_loss = self.kmeans(torch.cat((q, k), dim=2), update_kmeans)
             q_dists, k_dists = split_at_index(2, t, dists)
             indices = distribution(q_dists, wsz)
-            kv_indices = distribution(k_dists, c_wsz)
+            kv_indices = distribution(k_dists, kv_wsz)
+        else:
+            dists, aux_loss = self.kmeans(q, update_kmeans)
+            k = F.normalize(k, dim=-1).to(q)
+            indices = distribution(dists, wsz)
+            kv_indices = indices
 
         q = batched_index_select(q, indices)
         k = batched_index_select(k, kv_indices)
@@ -405,6 +296,9 @@ class KmeansAttention(nn.Module):
         reshape_with_window = lambda x: x.reshape(b, h, nc, -1, d)
         q, k, v = map(reshape_with_window, (q, k, v))
 
+        m_k, m_v = map(lambda x: expand_dim(x, 0, b).to(q), (self.mem_key, self.mem_value))
+        k, v = map(lambda x: torch.cat(x, dim=3), ((m_k, k), (m_v, v)))
+
         dots = torch.einsum('bhnid,bhnjd->bhnij', q, k) * (d ** -0.5)
 
         mask_value = max_neg_value(dots)
@@ -417,16 +311,21 @@ class KmeansAttention(nn.Module):
             kv_mask = expand_dim(key_mask, 1, h).gather(2, kv_indices)
             q_mask, kv_mask = map(lambda t: t.reshape(b, h, nc, -1), (q_mask, kv_mask))
             mask = q_mask[:, :, :, :, None] * kv_mask[:, :, :, None, :]
+            mask = F.pad(mask, (self.num_mem_kv, 0), value=True)
             dots.masked_fill_(~mask, mask_value)
             del mask
 
         if self.causal:
-            mask = torch.ones(wsz, wsz, device=device).byte().triu_(1).bool()
-            dots.masked_fill_(mask, mask_value)
-            del mask
+            q_mask, kv_mask = map(lambda t: t.reshape(b, h, nc, -1), (indices, kv_indices))
+            mask = q_mask[:, :, :, :, None] >= kv_mask[:, :, :, None, :]
+            mask = F.pad(mask, (self.num_mem_kv, 0), value=True)
+            dots.masked_fill_(~mask, mask_value)
+            del mask            
 
         if self.shared_qk:
-            mask = torch.eye(wsz, device=dots.device).bool()
+            q_mask, kv_mask = map(lambda t: t.reshape(b, h, nc, -1), (indices, kv_indices))
+            mask = q_mask[:, :, :, :, None] == kv_mask[:, :, :, None, :]
+            mask = F.pad(mask, (self.num_mem_kv, 0), value=False)
             dots.masked_fill_(mask, TOKEN_SELF_ATTN_VALUE)
             del mask
 
@@ -472,7 +371,7 @@ class FeedForward(nn.Module):
 # self attention
 
 class SelfAttention(nn.Module):
-    def __init__(self,  dim, depth, max_seq_len, heads, local_attn_heads, window_size, local_attn_window_size = None, causal = False, attn_dropout = 0., dropout = 0., kmeans_ema_decay = 0.999, commitment_factor = 1e-4, receives_context = False, context_window_size = None, rel_pos_emb = True):
+    def __init__(self,  dim, depth, max_seq_len, heads, local_attn_heads, window_size, dim_head = None, local_attn_window_size = None, causal = False, attn_dropout = 0., dropout = 0., kmeans_ema_decay = 0.999, commitment_factor = 1e-4, receives_context = False, context_window_size = None, rel_pos_emb = True, num_mem_kv = 0, shared_qk = False):
         super().__init__()
         assert (dim % heads) == 0, 'hidden dimension must be divisible by number of heads'
         assert (max_seq_len % window_size) == 0, 'maximum sequence length must be divisible by the target window size'
@@ -483,6 +382,7 @@ class SelfAttention(nn.Module):
         local_attn_window_size = default(local_attn_window_size, window_size // 2)
         context_window_size = default(context_window_size, window_size)
 
+        self.shared_qk = shared_qk
         self.receives_context = receives_context
         self.heads = heads
         self.local_attn_heads = local_attn_heads
@@ -490,47 +390,54 @@ class SelfAttention(nn.Module):
 
         self.window_size = window_size
 
-        head_dim = dim // heads
+        dim_head = default(dim_head, dim // heads)
+        dim_heads = dim_head * heads
+        self.dim_head = dim_head
+
         num_clusters = max_seq_len // window_size
 
         if self.local_attn_heads > 0:
-            self.local_attn = LocalAttention(local_attn_window_size, local_attn_heads, head_dim, causal = True, shared_qk = True, dropout = attn_dropout, rel_pos_emb = rel_pos_emb)
+            rel_pos_emb_config = (dim_head, local_attn_heads) if rel_pos_emb else None
+            self.local_attn = LocalAttention(local_attn_window_size, causal = True, dropout = attn_dropout, rel_pos_emb_config = rel_pos_emb_config, shared_qk = shared_qk)
 
         if self.global_attn_heads > 0:
-            self.global_attn = KmeansAttention(num_clusters, window_size, self.global_attn_heads, head_dim, causal = causal, dropout = attn_dropout, ema_decay = kmeans_ema_decay, commitment = commitment_factor, shared_qk = not receives_context)
+            self.global_attn = KmeansAttention(num_clusters, window_size, self.global_attn_heads, dim_head, causal = causal, dropout = attn_dropout, ema_decay = kmeans_ema_decay, commitment = commitment_factor, receives_context = receives_context, num_mem_kv = num_mem_kv, shared_qk = shared_qk)
 
-        self.to_qk = nn.Linear(dim, dim, bias = False)
-        self.to_v = nn.Linear(dim, dim, bias = False)
-        self.to_out = nn.Linear(dim, dim, bias = False)
+        self.to_q = nn.Linear(dim, dim_heads, bias = False)
+        self.to_v = nn.Linear(dim, dim_heads, bias = False)
+        self.to_out = nn.Linear(dim_heads, dim, bias = False)
+
+        if not self.shared_qk:
+            self.to_k = nn.Linear(dim, dim_heads, bias = False)
 
         self.dropout = nn.Dropout(dropout)
 
     def forward(self, x, context = None, input_mask = None, context_mask = None, **kwargs):
         assert not (self.receives_context and context is None), 'context must be passed if self attention is set to receive context'
-        b, t, e, h = *x.shape, self.heads
-        head_dim = e // self.heads
+        b, t, e, h, dh = *x.shape, self.heads, self.dim_head
 
-        split_heads = lambda v: v.reshape(b, -1, h, head_dim).transpose(1, 2).contiguous()
+        split_heads = lambda v: v.reshape(b, -1, h, dh).transpose(1, 2).contiguous()
 
-        if not self.receives_context:
-            qk, v = self.to_qk(x), self.to_v(x)
+        kv_input = x if not self.receives_context else context
 
-            qk, v = map(split_heads, (qk, v))
-            split_index_fn = partial(split_at_index, 1, self.local_attn_heads)
-            (lqk, qk), (lv, v) = map(split_index_fn, (qk, v))
-            has_local, has_global = map(lambda x: x.shape[1] > 0, (lqk, qk))
+        q, v = self.to_q(x), self.to_v(kv_input)
 
-            q, k, v = qk, qk, v
+        if not self.shared_qk:
+            k = self.to_k(kv_input)
         else:
-            q, k, v = self.to_qk(x), self.to_qk(context), self.to_v(context)
-            q, k, v = map(split_heads, (q, k, v))
-            has_local, has_global = False, True
+            k = self.to_q(kv_input) if self.receives_context else q
+
+        q, k, v = map(split_heads, (q, k, v))
+
+        split_index_fn = partial(split_at_index, 1, self.local_attn_heads)
+        (lq, q), (lk, k), (lv, v) = map(split_index_fn, (q, k, v))
+        has_local, has_global = map(lambda x: x.shape[1] > 0, (lq, q))
 
         out = []
         total_loss = torch.tensor(0., requires_grad=True, **to(x))
 
         if has_local:
-            local_out = self.local_attn(lqk, lqk, lv, input_mask = input_mask)
+            local_out = self.local_attn(lq, lk, lv, input_mask = input_mask)
             out.append(local_out)
 
         if has_global:
@@ -545,8 +452,10 @@ class SelfAttention(nn.Module):
         return self.dropout(out), total_loss
 
 class RoutingTransformer(nn.Module):
-    def __init__(self, dim, depth, max_seq_len, heads = 8, window_size = 64, local_attn_window_size = None, causal = False, weight_tie = False, attn_dropout = 0., ff_dropout = 0., attn_layer_dropout = 0., layer_dropout = 0., n_local_attn_heads = 0, ff_glu = False, reversible = False, ff_chunks = 1, kmeans_ema_decay = 0.999, commitment_factor = 1e-4, receives_context = False, context_window_size = None, _register_kmeans_update = False, rel_pos_emb = True, pkm_layers = tuple(), pkm_num_keys = 128):
+    def __init__(self, dim, depth, max_seq_len, heads = 8, dim_head = None, window_size = 64, local_attn_window_size = None, causal = False, weight_tie = False, attn_dropout = 0., ff_dropout = 0., attn_layer_dropout = 0., layer_dropout = 0., n_local_attn_heads = 0, ff_glu = False, reversible = False, ff_chunks = 1, kmeans_ema_decay = 0.999, commitment_factor = 1e-4, receives_context = False, context_window_size = None, _register_kmeans_update = False, rel_pos_emb = True, pkm_layers = tuple(), pkm_num_keys = 128, moe_layers = tuple(), moe_num_experts = 4, moe_loss_coef = 1e-2, num_mem_kv = 0, shared_qk = None, context_shared_qk = False):
         super().__init__()
+        shared_qk = default(shared_qk, causal) # default to shared qk when causal, due to experimental results
+
         local_attn_window_size = default(local_attn_window_size, window_size // 2)
         if type(n_local_attn_heads) is not tuple:
             n_local_attn_heads = tuple([n_local_attn_heads] * depth)
@@ -557,20 +466,24 @@ class RoutingTransformer(nn.Module):
         layers = nn.ModuleList([])
         fn_wrapper = partial(PreNorm, dim)
 
-        get_attn = lambda local_heads: SelfAttention(dim, depth, max_seq_len, heads, local_heads, window_size, causal = causal, local_attn_window_size = local_attn_window_size, attn_dropout = attn_dropout, dropout = attn_layer_dropout, kmeans_ema_decay = kmeans_ema_decay, commitment_factor = commitment_factor, rel_pos_emb = rel_pos_emb)
+        get_attn = lambda local_heads: SelfAttention(dim, depth, max_seq_len, heads, local_heads, window_size, causal = causal, dim_head = dim_head, local_attn_window_size = local_attn_window_size, attn_dropout = attn_dropout, dropout = attn_layer_dropout, kmeans_ema_decay = kmeans_ema_decay, commitment_factor = commitment_factor, rel_pos_emb = rel_pos_emb, num_mem_kv = num_mem_kv, shared_qk = shared_qk)
         get_ff = lambda: Chunk(ff_chunks, FeedForward(dim, dropout = ff_dropout, glu = ff_glu), along_dim=1)
-        get_context_attn = lambda: SelfAttention(dim, depth, max_seq_len, heads, 0, window_size, local_attn_window_size = local_attn_window_size, attn_dropout = attn_dropout, dropout = attn_layer_dropout, kmeans_ema_decay = kmeans_ema_decay, commitment_factor = commitment_factor, receives_context = True, context_window_size = context_window_size)
+        get_context_attn = lambda: SelfAttention(dim, depth, max_seq_len, heads, 0, window_size, dim_head = dim_head, local_attn_window_size = local_attn_window_size, attn_dropout = attn_dropout, dropout = attn_layer_dropout, kmeans_ema_decay = kmeans_ema_decay, commitment_factor = commitment_factor, receives_context = True, context_window_size = context_window_size, num_mem_kv = num_mem_kv, shared_qk = context_shared_qk)
         get_context_ff = lambda: Chunk(ff_chunks, FeedForward(dim, dropout = ff_dropout, glu = ff_glu), along_dim=1)
-        get_pkm = lambda: PKM(dim, num_keys = pkm_num_keys, use_evonorm = False)
+        get_pkm = lambda: PKM(dim, num_keys = pkm_num_keys)
+        get_moe = lambda: MoE(dim, num_experts = moe_num_experts, loss_coef = moe_loss_coef)
 
         if weight_tie:
             assert len(set(n_local_attn_heads)) == 1, 'you can only weight tie if number of local attention heads for all layers is the same'
-            get_attn, get_ff, get_context_attn, get_context_ff, get_pkm = map(cache_fn, (get_attn, get_ff, get_context_attn, get_context_ff, get_pkm))
+            get_attn, get_ff, get_context_attn, get_context_ff, get_pkm, get_moe = map(cache_fn, (get_attn, get_ff, get_context_attn, get_context_ff, get_pkm, get_moe))
 
         for ind, local_heads in zip(range(depth), n_local_attn_heads):
             layer = ind + 1
-            use_ff = layer not in cast_tuple(pkm_layers)
-            get_parallel_fn = get_ff if use_ff else get_pkm
+            use_pkm = layer in cast_tuple(pkm_layers)
+            use_moe = layer in cast_tuple(moe_layers)
+
+            get_parallel_fn = get_pkm if use_pkm else get_ff
+            get_parallel_fn = get_moe if use_moe else get_parallel_fn
 
             layers.append(nn.ModuleList([
                 fn_wrapper(get_attn(local_heads)),
@@ -606,7 +519,7 @@ class RoutingTransformer(nn.Module):
         return x, loss
 
 class RoutingTransformerLM(nn.Module):
-    def __init__(self, num_tokens, dim, depth, max_seq_len, heads = 8, window_size = 64, local_attn_window_size = None, causal = False, emb_dim = None, weight_tie = False, attn_dropout = 0., ff_dropout = 0., attn_layer_dropout = 0., layer_dropout = 0., ff_mult = 4, ff_activation = None, ff_glu = False, return_embeddings = False, n_local_attn_heads = 0, reversible = False, ff_chunks = 1, kmeans_ema_decay = 0.999, commitment_factor = 1e-4, receives_context = False, context_window_size = None, rel_pos_emb = True, _register_kmeans_update = True, pkm_layers = tuple(), pkm_num_keys = 128):
+    def __init__(self, num_tokens, dim, depth, max_seq_len, heads = 8, dim_head = None, window_size = 64, local_attn_window_size = None, causal = False, emb_dim = None, weight_tie = False, attn_dropout = 0., ff_dropout = 0., attn_layer_dropout = 0., layer_dropout = 0., ff_mult = 4, ff_activation = None, ff_glu = False, return_embeddings = False, n_local_attn_heads = 0, reversible = False, ff_chunks = 1, kmeans_ema_decay = 0.999, commitment_factor = 1e-4, receives_context = False, context_window_size = None, rel_pos_emb = True, _register_kmeans_update = True, pkm_layers = tuple(), pkm_num_keys = 128, moe_layers = tuple(), moe_num_experts = 4, moe_loss_coef = 1e-2, num_mem_kv = 0, shared_qk = None, context_shared_qk = False):
         super().__init__()
         assert (max_seq_len % window_size) == 0, 'max sequence length must be divisible by the window size, to calculate number of kmeans cluster'
         emb_dim = default(emb_dim, dim)
@@ -614,7 +527,7 @@ class RoutingTransformerLM(nn.Module):
 
         self.token_emb = nn.Embedding(num_tokens, emb_dim)
         self.axial_pos_emb = AxialPositionalEmbedding(emb_dim, axial_shape=(max_seq_len // window_size, window_size))
-        self.routing_transformer = RoutingTransformer(dim, depth, max_seq_len, heads = heads, window_size = window_size, local_attn_window_size = local_attn_window_size, causal = causal, weight_tie = weight_tie, ff_dropout = ff_dropout, attn_dropout = attn_dropout, attn_layer_dropout = attn_layer_dropout, layer_dropout = layer_dropout, n_local_attn_heads = n_local_attn_heads, ff_glu = ff_glu, reversible = reversible, ff_chunks = ff_chunks, kmeans_ema_decay = kmeans_ema_decay, receives_context = receives_context, context_window_size = context_window_size, rel_pos_emb = rel_pos_emb, pkm_layers = pkm_layers, pkm_num_keys = pkm_num_keys, _register_kmeans_update = _register_kmeans_update)
+        self.routing_transformer = RoutingTransformer(dim, depth, max_seq_len, heads = heads, dim_head = dim_head, window_size = window_size, local_attn_window_size = local_attn_window_size, causal = causal, weight_tie = weight_tie, ff_dropout = ff_dropout, attn_dropout = attn_dropout, attn_layer_dropout = attn_layer_dropout, layer_dropout = layer_dropout, n_local_attn_heads = n_local_attn_heads, ff_glu = ff_glu, reversible = reversible, ff_chunks = ff_chunks, kmeans_ema_decay = kmeans_ema_decay, receives_context = receives_context, context_window_size = context_window_size, rel_pos_emb = rel_pos_emb, pkm_layers = pkm_layers, pkm_num_keys = pkm_num_keys,  moe_layers = moe_layers, moe_num_experts = moe_num_experts, moe_loss_coef = moe_loss_coef, num_mem_kv = num_mem_kv, shared_qk = shared_qk, context_shared_qk = context_shared_qk, _register_kmeans_update = _register_kmeans_update)
 
         if emb_dim != dim:
             self.routing_transformer = ProjectInOut(self.routing_transformer, emb_dim, dim, project_out = not return_embeddings)
